--[[

	UI element creation/tracking for EnergyWatch

--]]


--[[ Globals --]]
EnergyWatchUI = {}

--[[ Main addon frame creation and main local vars ]]--

function EnergyWatchUI.CreateEnergyBar()
	local energyBar = CreateFrame("Frame", "EnergyWatchBar", UIParent)
	energyBar:SetMovable(true)
	energyBar:EnableMouse(true)
	energyBar:SetSize(116, 26)

	energyBar:SetScript("OnMouseDown", function (self, button)
						if button == "LeftButton" and not EnergyWatch.GetConfigValue("locked") then
 							self:StartMoving();
 						end
	end)
	energyBar:SetScript("OnMouseUp", function (self, button)
						if button == "LeftButton" then
 							self:StopMovingOrSizing();
 						end
	end)
	energyBar:SetPoint("CENTER", 0, 80)
	energyBar:Hide()

	local energyBarBackground = energyBar:CreateTexture("EnergyWatchBarBackground", "BACKGROUND")
	energyBarBackground:SetSize(107, 13)
	energyBarBackground:SetTexture(0, 0, 0, .5)
	energyBarBackground:SetPoint("TOP", 0, -2)

	local energyBarBorder = energyBar:CreateTexture("EnergyWatchBarBorder", "OVERLAY")
	energyBarBorder:SetSize(111, 20)
	energyBarBorder:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border")
	energyBarBorder:SetPoint("TOP", 0, 2)

	local energyBarText = energyBar:CreateFontString("EnergyWatchText", "OVERLAY", "GameFontHighlight")
	energyBarText:SetSize(110, 16)
	energyBarText:SetPoint("TOP", 0, 0)

	local energyBarStatus = CreateFrame("StatusBar", "EnergyWatchStatusBar", energyBar)
	energyBarStatus:SetSize(105, 13)
	energyBarStatus:SetPoint("TOP", 0, -2)
	--print("status bar is at level: " .. energyBarStatus:GetFrameLevel())
	--energyBarStatus:SetFrameLevel(1)
	energyBarStatus:SetFrameLevel(energyBarStatus:GetParent():GetFrameLevel())
	--print("status bar is at level: " .. energyBarStatus:GetFrameLevel())

	local energyBarStatusTexture = energyBarStatus:CreateTexture("EnergyWatchStatusBarTexture", "BACKGROUND")
	energyBarStatusTexture:SetTexture("Interface\\TargetingFrame\\UI-StatusBar")

	energyBarStatus:SetStatusBarTexture(energyBarStatusTexture)
	--energyBarStatus:SetFrameLevel(1)
	--energyBarStatus:SetStatusBarColor(1, 1, 0)
	--print("status bar is at level: " .. energyBarStatus:GetFrameLevel())
	--energyBarStatus:SetFrameLevel(1)
	--print("status bar is at level: " .. energyBarStatus:GetFrameLevel())

end

--[[ Config UI --]]
function EnergyWatchUI.CreateConfigMenu()
	local configPanel = CreateFrame("Frame", "EnergyWatchConfigFrame", UIParent)
	configPanel.name = "EnergyWatch"
	configPanel.okay = function (self) return end
	configPanel.cancel = function (self) return end

	local addonName, addonTitle, addonNotes = GetAddOnInfo('EnergyWatch')
	local configPanelText = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	configPanelText:SetPoint('TOPLEFT', 16, -16)
	configPanelText:SetText(addonTitle)

	local configPanelDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	--configPanelDesc:SetHeight(32)
	configPanelDesc:SetPoint('TOPLEFT', configPanelText, 'BOTTOMLEFT', 0, -8)
	configPanelDesc:SetPoint('RIGHT', configPanel, -32, 0)
	configPanelDesc:SetNonSpaceWrap(true)
	configPanelDesc:SetJustifyH('LEFT')
	configPanelDesc:SetJustifyV('TOP')
	configPanelDesc:SetText(addonNotes)

	local showOptionDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	showOptionDesc:SetPoint('TOPLEFT', configPanelDesc, 'BOTTOMLEFT', -2, -10)
	showOptionDesc:SetText("When to show Energy Watch")

	local showAlways = EnergyWatchUI.CreateCheckButton("Always", configPanel,"Always show Energy Watch\n(Bar will only show if your character is currently using Energy or Focus)", "showAlways", 'InterfaceOptionsCheckButtonTemplate')
	showAlways:SetPoint('TOPLEFT', showOptionDesc, 'BOTTOMLEFT', 0, 0)

	local showStealth = EnergyWatchUI.CreateCheckButton("In Stealth", configPanel,"Show Energy Watch while stealthed", "showStealth", 'InterfaceOptionsCheckButtonTemplate')
	showStealth:SetPoint('TOPLEFT', showAlways, 'BOTTOMLEFT', 10, 0)
	EnergyWatchUI.SetupInverseDependentControl(showAlways, showStealth)

	local showCombat = EnergyWatchUI.CreateCheckButton("In Combat", configPanel,"Show Energy Watch while in combat", "showCombat", 'InterfaceOptionsCheckButtonTemplate')
	showCombat:SetPoint('TOPLEFT', showStealth, 'BOTTOMLEFT', 0, 0)
	EnergyWatchUI.SetupInverseDependentControl(showAlways, showCombat)

	local textEditDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	textEditDesc:SetPoint('TOPLEFT', showCombat, 'BOTTOMLEFT', -10, -10)
	textEditDesc:SetText("Bar Text: ")

	local textEditBox = CreateFrame("EditBox",  configPanel:GetName() .. "BarTextEditBox", configPanel, "InputBoxTemplate")
	textEditBox:SetAutoFocus(false)
	textEditBox:SetSize(200, 32)
	textEditBox:SetMaxLetters(128)
	textEditBox:SetText(EnergyWatch.GetConfigValue("barText"))
	textEditBox:SetCursorPosition(0)
	textEditBox:SetScript("OnTextChanged", EnergyWatchUI.BarTextEditBoxChanged)
	textEditBox:SetPoint('TOPLEFT', textEditDesc, 'BOTTOMLEFT', 10, 0)

	local textEditHint = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	textEditHint:SetPoint('TOPLEFT', textEditBox, 'BOTTOMLEFT', 0, 0)
	textEditHint:SetText("&e = Energy, &em = Max Energy, &c = Combo Points")

	local barAlphaDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	barAlphaDesc:SetPoint('TOPLEFT', textEditHint, 'BOTTOMLEFT', -10, -10)
	barAlphaDesc:SetText("Bar Alpha (Opacity): ")

	local barAlphaSlider = CreateFrame("Slider", configPanel:GetName() .. "BarAlphaSlider", configPanel, "OptionsSliderTemplate")
	barAlphaSlider:SetMinMaxValues(0.0, 1.0)
	barAlphaSlider:SetValueStep(0.01)
	barAlphaSlider:SetValue(EnergyWatch.GetConfigValue("barAlpha"))
	_G[barAlphaSlider:GetName() .. "High"]:SetText("1.0")
	_G[barAlphaSlider:GetName() .. "Low"]:SetText("0.0")
	barAlphaSlider:SetScript("OnValueChanged", EnergyWatchUI.BarAlphaSliderChanged)
	barAlphaSlider:SetPoint('TOPLEFT', barAlphaDesc, 'BOTTOMLEFT', 10, 0)

	local barScaleDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	barScaleDesc:SetPoint('TOPLEFT', barAlphaSlider, 'BOTTOMLEFT', -10, -10)
	barScaleDesc:SetText("Bar Scale: ")

	local barScaleSlider = CreateFrame("Slider", configPanel:GetName() .. "barScaleSlider", configPanel, "OptionsSliderTemplate")
	barScaleSlider:SetMinMaxValues(.25, 3.0)
	barScaleSlider:SetValueStep(0.01)
	barScaleSlider:SetValue(EnergyWatch.GetConfigValue("barScale"))
	_G[barScaleSlider:GetName() .. "High"]:SetText("3.0")
	_G[barScaleSlider:GetName() .. "Low"]:SetText("0.25")
	barScaleSlider:SetScript("OnValueChanged", EnergyWatchUI.BarScaleSliderChanged)
	barScaleSlider:SetPoint('TOPLEFT', barScaleDesc, 'BOTTOMLEFT', 10, 0)

	InterfaceOptions_AddCategory(configPanel)
end

function EnergyWatchUI.BarTextEditBoxChanged(self, isUserInput)
	if isUserInput then
		EnergyWatch.SetConfigValue("barText", self:GetText())
		EnergyWatch.UpdateBar()
	end
end

function EnergyWatchUI.BarAlphaSliderChanged(self, value)
	EnergyWatch.SetConfigValue("barAlpha", value)
	EnergyWatchBar:SetAlpha(value)
end

function EnergyWatchUI.BarScaleSliderChanged(self, value)
	EnergyWatch.SetConfigValue("barScale", value)
	EnergyWatchBar:SetScale(value)
end

function EnergyWatchUI.CreateCheckButton(name, parent, tooltipText, configKey, template)
	local button = CreateFrame('CheckButton', parent:GetName() .. name, parent, template)
	_G[button:GetName() .. 'Text']:SetText(name)
	button:SetChecked(EnergyWatch.GetConfigValue(configKey))
	button:SetScript('OnClick', function(self)
		if self:GetChecked() then
			EnergyWatch.SetConfigValue(configKey, true)
		else 
			EnergyWatch.SetConfigValue(configKey, false)
		end
		if ( self.dependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.dependentControls) do
					control:Enable()
				end
			else
				for _, control in pairs(self.dependentControls) do
					control:Disable()
				end
			end
		end
		if ( self.inverseDependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.inverseDependentControls) do
					control:Disable()
				end
			else
				for _, control in pairs(self.inverseDependentControls) do
					control:Enable()
				end
			end
		end
		EnergyWatch.ShowOrHideBar()

	end)
	button:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(tooltipText, nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	button:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	return button
end

function EnergyWatchUI.SetupDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end
	
	assert(control)
	
	dependency.dependentControls = dependency.dependentControls or {}
	tinsert(dependency.dependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Enable()
		else
			control:Disable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end

function EnergyWatchUI.SetupInverseDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end
	
	assert(control)
	
	dependency.inverseDependentControls = dependency.inverseDependentControls or {}
	tinsert(dependency.inverseDependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Disable()
		else
			control:Enable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end